/*============================================================================
	SQL Server Versions: 2012 11.0.3321
------------------------------------------------------------------------------
	Copyright (C) 2012 Joe Sack, SQLskills.com
	All rights reserved. 

	For more scripts and sample code, check out
		http://www.sqlskills.com/ 

	You may alter this code for your own *non-commercial* purposes. You may
	republish altered code as long as you give due credit. 

	THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF
	ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
	TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
	PARTICULAR PURPOSE.
============================================================================*/ 

USE [Credit];
GO

-- Index Scan + Predicate (NOT a seek predicate)
SELECT provider_no
FROM dbo.provider
WHERE provider_name = 'Prov. Famous Ne';

-- Now let's add an index on provider_name
CREATE NONCLUSTERED
INDEX provider_provider_name
ON provider(provider_name);
GO

-- Index Seek + Seek Predicate
SELECT provider_no
FROM dbo.provider
WHERE provider_name = 'Prov. Famous Ne';

-- Index Seek Predicate + Filter
-- Does it push down the filter?
SELECT region_no, COUNT(*)
FROM dbo.provider
WHERE region_no IN (5,6,7)
GROUP BY region_no
HAVING COUNT(*) > 30;
GO

-- Probe Residual example
CREATE TABLE #member_no_staging
	( staging_id int NOT NULL IDENTITY(1,1) 
		PRIMARY KEY CLUSTERED,
	  member_no bigint NOT NULL);
GO

INSERT #member_no_staging
	(member_no)
SELECT member_no
FROM dbo.member2;

-- Why is there a "probe residual"?
SELECT firstname, lastname
FROM dbo.member m1
INNER JOIN #member_no_staging m2 ON
	m1.member_no = m2.member_no; 

-- Comparing to the previous example
CREATE TABLE #member_no_staging_2
	( staging_id int NOT NULL IDENTITY(1,1) 
		PRIMARY KEY CLUSTERED,
	  member_no int NOT NULL);
GO

INSERT #member_no_staging_2
	(member_no)
SELECT member_no
FROM dbo.member2;

-- Execute together
SELECT firstname, lastname
FROM dbo.member m1
INNER JOIN #member_no_staging m2 ON
	m1.member_no = m2.member_no 

SELECT firstname, lastname
FROM dbo.member m1
INNER JOIN #member_no_staging_2 m2 ON
	m1.member_no = m2.member_no 

DROP TABLE #member_no_staging_2;
DROP TABLE #member_no_staging;