/*============================================================================
	SQL Server Versions: 2012 11.0.3321
------------------------------------------------------------------------------
	Copyright (C) 2012 Joe Sack, SQLskills.com
	All rights reserved. 

	For more scripts and sample code, check out
		http://www.sqlskills.com/ 

	You may alter this code for your own *non-commercial* purposes. You may
	republish altered code as long as you give due credit. 

	THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF
	ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
	TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
	PARTICULAR PURPOSE.
============================================================================*/ 

USE Credit;
GO

-- Show charge entries where the charge
-- for previous dates exceeds the 
-- historical average, for a specific
-- member and category
-- (Takes about 30 seconds to run)

SET STATISTICS IO ON;

SELECT	c1.charge_no,
		c1.category_no,
		c1.charge_dt,
		c1.member_no,
		c1.charge_amt
FROM dbo.charge c1
WHERE c1.charge_amt >
(SELECT AVG(c2.charge_amt)
 FROM dbo.charge c2
 WHERE c2.charge_dt < 
		c1.charge_dt) AND
c1.member_no = 9620 AND
c1.category_no IN (2,3)
OPTION (MAXDOP 1, RECOMPILE);

SET STATISTICS IO OFF;

/*
Table 'Worktable'. Scan count 80, logical reads 5097533, physical reads 0, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.
Table 'charge'. Scan count 2, logical reads 18670, physical reads 0, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.
*/
-- What was the cost?



-- But are spools "BAD" in this case?  

-- How many executions for the index scan?
SET STATISTICS IO ON;

SELECT	c1.charge_no,
		c1.category_no,
		c1.charge_dt,
		c1.member_no,
		c1.charge_amt
FROM dbo.charge c1
WHERE c1.charge_amt >
(SELECT AVG(c2.charge_amt)
 FROM dbo.charge c2
 WHERE c2.charge_dt < 
		c1.charge_dt) AND
c1.member_no = 9620 AND
c1.category_no IN (2,3)
OPTION (MAXDOP 1, RECOMPILE, QUERYRULEOFF BuildSpool);

SET STATISTICS IO OFF;

/*

Table 'charge'. Scan count 81, logical reads 756135, physical reads 0, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.

vs.

Table 'Worktable'. Scan count 80, logical reads 5097533, physical reads 0, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.
Table 'charge'. Scan count 2, logical reads 18670, physical reads 0, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.

*/

-- On this subject - a technique for plan regression before/after
-- Plan Regression
SELECT [name],
        [promise_total],
        [promise_avg],
        [promised],
        [built_substitute],
        [succeeded]
INTO #t1
FROM sys.dm_exec_query_transformation_stats;

SELECT	TOP 1 c1.charge_no,
		c1.category_no,
		c1.charge_dt,
		c1.member_no,
		c1.charge_amt
FROM dbo.charge c1 
OPTION (RECOMPILE);

SELECT [name],
        [promise_total],
        [promise_avg],
        [promised],
        [built_substitute],
        [succeeded]
INTO #t2
FROM sys.dm_exec_query_transformation_stats;

-- What rules were applied?
SELECT t1.[name],
        t1.[promise_total],
        t1.[promise_avg],
        t1.[promised],
        t1.[built_substitute],
        t1.[succeeded]
FROM #t1 AS t1
INNER JOIN #t2 AS t2 ON
	t1.name = t2.name
WHERE t1.succeeded <> t2.succeeded

DROP TABLE #t1;
DROP TABLE #t2;
