/*============================================================================
	SQL Server Versions: 2012 11.0.3321
------------------------------------------------------------------------------
	Copyright (C) 2012 Joe Sack, SQLskills.com
	All rights reserved. 

	For more scripts and sample code, check out
		http://www.sqlskills.com/ 

	You may alter this code for your own *non-commercial* purposes. You may
	republish altered code as long as you give due credit. 

	THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF
	ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
	TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
	PARTICULAR PURPOSE.
============================================================================*/ 

USE [Credit];
GO


-- Clustered Index Seek -> 1 Row
-- What's the Estimated I/O cost?
SELECT  [member].[lastname]
FROM [dbo].[member]
WHERE [member].[member_no] = 1;

-- Find the Page ID so we can check the cache
SELECT  [lastname],
		sys.fn_PhysLocFormatter(%%physloc%%)
FROM [dbo].[member]
WHERE [member_no] = 1;

SELECT  [database_id],
        [file_id],
        [page_id],
        [page_level],
        [allocation_unit_id],
        [page_type],
        [row_count],
        [free_space_in_bytes],
        [is_modified],
        [numa_node],
        [read_microsec]
FROM    sys.dm_os_buffer_descriptors
WHERE   page_id = 248 AND
        database_id = DB_ID();

-- Since its cached, does cost change?
SELECT  [member].[lastname]
FROM    [dbo].[member]
WHERE   [member].[member_no] = 1;

-- Let's get it out of cache
DBCC DROPCLEANBUFFERS;

-- What about for a Clustered Index Scan?
-- What's the estimated I/O?
SELECT  [member].[member_no],
        [member].[lastname]
FROM    [dbo].[member];

-- How many pages in the table?
SELECT  [dm_db_partition_stats].[in_row_data_page_count]
FROM    [sys].[dm_db_partition_stats]
WHERE   [dm_db_partition_stats].[object_id] = OBJECT_ID('dbo.member') AND
        [dm_db_partition_stats].[index_id] = 1;

-- Compare the estimated I/O value to the plan
-- Random I/O - 0.003125
-- Sequential - 0.000740741
SELECT	0.003125 +
		0.000740741 * (142-1);

-- ****
-- Inflating the page count while maintaining the row count

-- Original
SELECT [member].[member_no],
 [member].[lastname]
 FROM [dbo].[member]
 OPTION (RECOMPILE);
 GO

-- Estimated CPU Cost = 0.011157
-- Estimated I/O Cost = 0.107569

-- Increase page count while maintaining row count
 UPDATE STATISTICS dbo.[member]
 WITH ROWCOUNT = 10000, PAGECOUNT = 149000;
 GO

 SELECT [member].[member_no],
 [member].[lastname]
 FROM [dbo].[member]
 OPTION (RECOMPILE);
 GO

-- Estimated CPU Cost = 0.011157 (same as previous 0.011157)
-- Estimated I/O Cost = 110.373 (increased from 0.107569)

 -- ***
 -- Inflating the row count while maintaining the page count
 UPDATE STATISTICS dbo.[member]
 WITH ROWCOUNT = 100000000, PAGECOUNT = 142;

 SELECT [member].[member_no],
 [member].[lastname]
 FROM [dbo].[member]
 OPTION (RECOMPILE);
 GO

-- Estimated CPU Cost = 110 (same as previous 0.011157)
-- Estimated I/O Cost = 0.107569 (no change from the original estimate with 142 data pages and 10,000 rows)

 -- What about influence on memory grants?

 UPDATE STATISTICS dbo.[member]
 WITH ROWCOUNT = 10000, PAGECOUNT = 142;

 SELECT  [member].[member_no],
 [member].[lastname],
 [curr_balance]
 FROM    [dbo].[member]
 ORDER BY [curr_balance] ASC
 OPTION  (RECOMPILE, MAXDOP 1);

  /*
<MemoryGrantInfo SerialRequiredMemory="512" SerialDesiredMemory="1904" RequiredMemory="512" DesiredMemory="1904" RequestedMemory="1904" GrantWaitTime="0" GrantedMemory="1904" MaxUsedMemory="680" />
*/

 -- ** 
 -- Inflating the page count while maintaining the row count
 UPDATE STATISTICS dbo.[member]
 WITH ROWCOUNT = 10000, PAGECOUNT = 149000;

  SELECT  [member].[member_no],
 [member].[lastname],
 [curr_balance]
 FROM    [dbo].[member]
 ORDER BY [curr_balance] ASC
 OPTION  (RECOMPILE, MAXDOP 1);

 /* No change!
 <MemoryGrantInfo SerialRequiredMemory="512" SerialDesiredMemory="1904" RequiredMemory="512" DesiredMemory="1904" RequestedMemory="1904" GrantWaitTime="0" GrantedMemory="1904" MaxUsedMemory="680" />
 */

 -- Inflating the row count while maintaining the page count
 UPDATE STATISTICS dbo.[member]
 WITH ROWCOUNT = 100000000, PAGECOUNT = 142;

   SELECT  [member].[member_no],
 [member].[lastname],
 [curr_balance]
 FROM    [dbo].[member]
 ORDER BY [curr_balance] ASC
 OPTION  (RECOMPILE, MAXDOP 1);

 /* the Sort operator CPU and I/O cost is sensitive to row counts

 <MemoryGrantInfo SerialRequiredMemory="512" SerialDesiredMemory="8564592" RequiredMemory="512" DesiredMemory="8564592" RequestedMemory="533744" GrantWaitTime="0" GrantedMemory="533744" MaxUsedMemory="680" />

 */

-- Restore database back to original
USE [master]
GO

RESTORE DATABASE [Credit] FROM  DISK = N'C:\Temp\ORIGINAL_CreditBackup100.bak'
 WITH  FILE = 1,  NOUNLOAD,  STATS = 5

GO


