/*============================================================================
	SQL Server Versions: 2012 11.0.3321
------------------------------------------------------------------------------
	Copyright (C) 2012 Joe Sack, SQLskills.com
	All rights reserved. 

	For more scripts and sample code, check out
		http://www.sqlskills.com/ 

	You may alter this code for your own *non-commercial* purposes. You may
	republish altered code as long as you give due credit. 

	THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF
	ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
	TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
	PARTICULAR PURPOSE.
============================================================================*/ 

/* Prep script

USE [AdventureWorksDW2012];
GO

-- Increase size of table to 3,865,472 rows
INSERT  [dbo].[FactInternetSales]
        ( [ProductKey] ,
          [OrderDateKey] ,
          [DueDateKey] ,
          [ShipDateKey] ,
          [CustomerKey] ,
          [PromotionKey] ,
          [CurrencyKey] ,
          [SalesTerritoryKey] ,
          [SalesOrderNumber] ,
          [SalesOrderLineNumber] ,
          [RevisionNumber] ,
          [OrderQuantity] ,
          [UnitPrice] ,
          [ExtendedAmount] ,
          [UnitPriceDiscountPct] ,
          [DiscountAmount] ,
          [ProductStandardCost] ,
          [TotalProductCost] ,
          [SalesAmount] ,
          [TaxAmt] ,
          [Freight] ,
          [CarrierTrackingNumber] ,
          [CustomerPONumber] ,
          [OrderDate] ,
          [DueDate] ,
          [ShipDate]
        )
        SELECT  [FactInternetSales].[ProductKey] ,
                [FactInternetSales].[OrderDateKey] ,
                [FactInternetSales].[DueDateKey] ,
                [FactInternetSales].[ShipDateKey] ,
                [FactInternetSales].[CustomerKey] ,
                [FactInternetSales].[PromotionKey] ,
                [FactInternetSales].[CurrencyKey] ,
                [FactInternetSales].[SalesTerritoryKey] ,
                LEFT(CAST(NEWID() AS NVARCHAR(36)), 20) ,
                [FactInternetSales].[SalesOrderLineNumber] ,
                [FactInternetSales].[RevisionNumber] ,
                [FactInternetSales].[OrderQuantity] ,
                [FactInternetSales].[UnitPrice] ,
                [FactInternetSales].[ExtendedAmount] ,
                [FactInternetSales].[UnitPriceDiscountPct] ,
                [FactInternetSales].[DiscountAmount] ,
                [FactInternetSales].[ProductStandardCost] ,
                [FactInternetSales].[TotalProductCost] ,
                [FactInternetSales].[SalesAmount] ,
                [FactInternetSales].[TaxAmt] ,
                [FactInternetSales].[Freight] ,
                [FactInternetSales].[CarrierTrackingNumber] ,
                [FactInternetSales].[CustomerPONumber] ,
                [FactInternetSales].[OrderDate] ,
                [FactInternetSales].[DueDate] ,
                [FactInternetSales].[ShipDate]
        FROM    [dbo].[FactInternetSales];
GO 3 -- Executes this INSERT 3 separate times

-- Setting all Fact table rows to the Yen currency
UPDATE [dbo].[FactInternetSales]
SET [CurrencyKey] = 102;
GO

*/

USE [AdventureWorksDW2012];
GO


-- Investigate rows by currency
SELECT  [c].[CurrencyName],
       COUNT(*) AS [RowCount]
FROM    [dbo].[FactInternetSales] AS [fis]
        INNER JOIN [dbo].[DimCurrency] AS [c]
        ON [fis].[CurrencyKey] = [c].[currencykey]
GROUP BY [c].[CurrencyName]
ORDER BY [c].[CurrencyName];
GO  

-- No rows for the Pakistan Rupee - so what plan do we see?
SET STATISTICS IO ON;

SELECT  [od].[CalendarYear] ,
        [p].[ProductLine] ,
        SUM([fis].[SalesAmount]) AS [SalesAMT]
FROM    [dbo].[FactInternetSales] AS [fis]
        INNER JOIN [dbo].[DimProduct] AS [p]
        ON [fis].[ProductKey] = [p].[ProductKey]
        INNER JOIN [dbo].[DimCurrency] AS [c]
        ON [fis].[CurrencyKey] = [c].[currencykey]
        INNER JOIN [dbo].[DimDate] AS [od]
        ON [fis].[OrderDateKey] = [od].[DateKey]
WHERE   [c].[CurrencyName] = N'Pakistan Rupee'
GROUP BY [od].[CalendarYear] ,
        [p].[ProductLine]
ORDER BY [od].[CalendarYear] ,
        [p].[ProductLine]
OPTION (RECOMPILE);
GO 

SET STATISTICS IO OFF;

-- What was the memory grant?
/*
<MemoryGrantInfo SerialRequiredMemory="3072" SerialDesiredMemory="6416" RequiredMemory="14112" DesiredMemory="17472" RequestedMemory="17472" GrantWaitTime="0" GrantedMemory="17472" MaxUsedMemory="5784" />
*/

-- Option - predicate on fact table
-- CurrencyKey 75 = Pakistan Rupee

SET STATISTICS IO ON;
SELECT  [od].[CalendarYear] ,
        [p].[ProductLine] ,
        SUM([fis].[SalesAmount]) AS [SalesAMT]
FROM    [dbo].[FactInternetSales] AS [fis]
        INNER JOIN [dbo].[DimProduct] AS [p]
        ON [fis].[ProductKey] = [p].[ProductKey]
        INNER JOIN [dbo].[DimDate] AS [od]
        ON [fis].[OrderDateKey] = [od].[DateKey]
WHERE   [fis].[CurrencyKey] = 75 -- predicate on fact table!
GROUP BY [od].[CalendarYear] ,
        [p].[ProductLine]
ORDER BY [od].[CalendarYear] ,
        [p].[ProductLine]
OPTION (RECOMPILE);

SET STATISTICS IO OFF;
GO 

-- Option - FORCESEEK
CREATE NONCLUSTERED INDEX IX_FIS_CurrencyKey_ProductKey_DateKey
ON [dbo].[FactInternetSales]([CurrencyKey], [ProductKey], [OrderDateKey])
INCLUDE ([SalesAmount]);
GO 

SET STATISTICS IO ON;
GO

SELECT  [od].[CalendarYear] ,
        [p].[ProductLine] ,
        SUM([fis].[SalesAmount]) AS [SalesAMT]
FROM    [dbo].[FactInternetSales] AS [fis] WITH (FORCESEEK)
        INNER JOIN [dbo].[DimProduct] AS [p]
        ON [fis].[ProductKey] = [p].[ProductKey]
        INNER JOIN [dbo].[DimCurrency] AS [c]
        ON [fis].[CurrencyKey] = [c].[currencykey]
        INNER JOIN [dbo].[DimDate] AS [od]
        ON [fis].[OrderDateKey] = [od].[DateKey]
WHERE   [c].[CurrencyName] = N'Pakistan Rupee'
GROUP BY [od].[CalendarYear] ,
        [p].[ProductLine]
ORDER BY [od].[CalendarYear] ,
        [p].[ProductLine];
GO 

SET STATISTICS IO OFF;
GO

-- Columnstore indexes (doesn't fix CE problem but it probably won't matter, super-fast)
-- Dynamic SQL